<?php
//Copyright (c) 2005, Game-Monitor.com (http://www.game-monitor.com/sdk/)
//All rights reserved.
//
//This software is licensed under the BSD license. 
//	http://www.game-monitor.com/sdk/license.php
//
//-------------------------------------------------------------------------
//This code will only work for 'premium' servers. Subscribe today and add 
//your servers to the premium server list.
//   http://www.game-monitor.com/subscribe.php
//-------------------------------------------------------------------------


require_once 'XML/Parser.php';

/**
 * Used to parse XML feeds into objects and arrays
 *
 * @author  Wojciech Hlibowicki <contact@game-monitor.com>
 *
 * @notes   - Requires PEAR's XML_Parser installed
 *          - Developed using PHP v4.3.11
 *
 * @package gm-xml
 */
class gm_xmlParse extends XML_Parser
{
	/**
	 * @var array/stack contains objects created during parsing
	 */
	var $stack;

	/**
	 * @var object containts the current object
	 */
	var $xmlObj;

	/**
	 * Creates an XML parser that creates objects corresponding to the XML feed
	 */
	function gm_xmlParse()
	{
		//setup variables and call parent constructor
		$this->stack=array();
		parent::XML_Parser();
		$this->xmlObj='';
	}

	/**
	 * Used by XML_Parser to handle the start XML tags <..> 
	 *
	 * @param not_used
	 * @param string 
	 * @param not_used
	 * @see $stack, $xmlObj
	 */
	function startHandler($xp, $name, $attribs)
	{	
		//push object to stack and reset current object
		array_push($this->stack,$this->xmlObj);
		$this->xmlObj='';
	}

	/**
	 * Used by XML_Parser to handle the end XML tags </..>
	 *
	 * @param not_used
	 * @param string
	 * @see $stack, $xmlObj
	 */
	function endHandler($xp, $name)
	{
		//pop object from the stack
		$o=array_pop($this->stack);

		//check if we need to construct an array (duplicate names?)
		if (isset($o->$name)) {
			//check if start of array
			if (sizeof($o->$name)==1) {
				$temp=$o->$name;
				unset($o->$name);

				$o->{$name}[0]=$temp;
				$o->{$name}[1]=$this->xmlObj;
			} else {
				//not the start of array, so tack on object to the end
				$o->{$name}[sizeof($o->$name)]=$this->xmlObj;
			}
		} else {
			//normal variable (not array, yet?)
			$o->$name=$this->xmlObj;
		}
		//set current object
		$this->xmlObj=$o;
	}

	/**
	 * Used by XML_Parser to handle the data in between XML tags <..>data</..>
	 *
	 * @param not_used
	 * @param string
	 * @see $xmlObj
	 */
	function cdataHandler($xp, $cdata)
	{
		//set current data if not empty
		if (trim($cdata)!='') 
			$this->xmlObj=$cdata;
	}
}

/**
 * Highly customizable Game-Monitor XML Parser and render for
 * images and HTML
 *
 * @author  Wojciech Hlibowicki <contact@game-monitor.com>
 *
 * @notes   - Requires PEAR's XML_Parser installed
 *          - Developed using PHP v4.3.11
 *          - All XML Feeds requires you to subscribe to
 *                      work
 *
 * @package gm-xml
 */
class gm_xml {
	/**
	 * @var string path to cache directory
	 */
	var $GM_CACHE_DIR;
	
	/**
	 * @var stirng path to data directory
	 */
	var $GM_DATA_DIR;
	
	/**
	 * @var int cache expiry time
	 */
	var $GM_CACHE_EXPIRY;
	

	/**
	 * Constructor for class, used to set settings
	 *
	 * @param string path to store cache files (must be writable by this php script)
	 * @param string path to data files
	 * @param int time in seconds after update for cache to expire 
	 */
	function gm_xml($cache_dir='gm_cache/', $data_dir='gm_data/', $cache_expiry=300) {
		//set default settings
		$this->GM_CACHE_DIR=e_PLUGIN."/ebfstats_menu/gm/".$cache_dir;	  	//directory to store cache files
		$this->GM_DATA_DIR=e_PLUGIN."/ebfstats_menu/gm/".$data_dir;	    	//directory to data files
		$this->GM_CACHE_EXPIRY=$cache_expiry;	//in number of seconds
	}

	/**
	 * Reads a whole file into a variable and returns it
	 *
	 * @access private
	 * @param string $filename filename to read
	 * @return string content of file
	 */
	function loadFile($filename) {
		//simple grab contents of file and return it, return an empty string on fail
		$txt='';
		$fp=@fopen($filename, "r");
	        if ($fp) {
        	        $txt=fread($fp, filesize($filename));
	                fclose($fp);
	        }
		return $txt;
	}

	/**
	 * Converts objects to 2 seperate arrays (from,to) for use with str_replace
	 *
	 * @access  private
	 * @param object object to convert
	 * @param string tag to append and prepend to the object names
	 * @param int recursion level (used to prevent function from 'running away')
	 * @return mixed containing two arrays (from,to)
	 */
	function convertObject($obj, $tag='%', $rlvl=0) {
		//prevent this recursive function of going out of control (just in case)
		if ($rlvl>3) return;

		//parse through each variable within the object
	        while (list($key, $value) = @each ($obj)) {
			if (is_object($value)) {
				//recursive call, tag on object name to all of the object variables
				//and only put the beggining/end tags on the upper level one
				$temp_v=$this->convertObject($value,'',$rlvl++);
				for ($i=0; $i<sizeof($temp_v->from); $i++) {
					$var->from[]=$tag.$key.'_'.$temp_v->from[$i].$tag;
					$var->to[]=$temp_v->to[$i];
				}
			} else if (is_array($value)) {
				//not converted
			} else {
				//later used in a call to str_replace
				$var->from[]=$tag.$key.$tag;
				$var->to[]=$value;
			}
		}
		return $var;
        }

	/**
	 * Checked if cache expired
	 *
	 * @param string filename to check
	 * @return bool
	 */
	function isCacheExpired($filename) {
		$cacheFile=$this->GM_CACHE_DIR.$filename;

		//determine how many seconds ago the file was modified
		$cacheMTime=time()-@filemtime($cacheFile);

		//check if file is too old and we should get a new one
		if ($cacheMTime>$this->GM_CACHE_EXPIRY) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Updates cache
	 *
	 * @param string filename
	 * @param string data
	 */
	function updateCache($filename, $data) {
		//write xml data to cache file
		$fp=fopen($this->GM_CACHE_DIR.$filename,'w');
		if ($fp) {
			fwrite($fp,$data);
			fclose($fp);
		} else {
			die('Unable to open cache file for writing: '.$cacheFile);
		}
	}


	/**
	 * Refreshes cache if needed (if expired)
	 *
	 * @param string ip address of game server
	 * @param int port number of game server
	 */
	function checkServerCache($ip, $gamePort) {
		$cacheFile=md5($ip.':'.$port).'-server.xml';

		if ($this->isCacheExpired($cacheFile)) {
			//get xml data from Game-Monitor.com
			$fp=@fopen('http://www.game-monitor.com/client/server-xml.php?ip='.$ip.':'.$gamePort,'r');
			if ($fp) {
				$xmlData='';
				while (!feof($fp)) {
					$xmlData.=fread($fp,40960);
				}
				fclose($fp);
			} else {
				die('Unable to contact Game-Monitor.com');
			}

			//write xml data to cache file
			$this->updateCache($cacheFile, $xmlData);
		}
	}

	/**
	 * Refreshes cache if needed (if expired)
	 *
	 * @param int user id
	 * @param int list id
	 */
	function checkBuddyCache($uid, $id=0) {
		$cacheFile=md5($uid.'.'.$id).'-buddy.xml';

		if ($this->isCacheExpired($cacheFile)) {
			//get xml data from Game-Monitor.com
			$fp=@fopen('http://www.game-monitor.com/client/buddyList.php?xml=1&pid=1&uid='.$uid.'&id='.$id,'r');
			if ($fp) {
				$xmlData='';
				while (!feof($fp)) {
					$xmlData.=fread($fp,40960);
				}
				fclose($fp);
			} else {
				die('Unable to contact Game-Monitor.com');
			}

			//write xml data to cache file
			$this->updateCache($cacheFile, $xmlData);
		}
        }

	/**
	 * Load XML file and return parsed object
	 *
	 * @param string ip address of game server
	 * @param int port number of game server
	 * @return mixed representing XML data
	 */
	function grabServerObject($ip, $gamePort) {
		//load/download the xml file and parse
		$p = &new gm_xmlParse();

		$this->checkServerCache($ip, $gamePort);
		$p->setInputFile($this->GM_CACHE_DIR.md5($ip.':'.$port).'-server.xml');
		$result=$p->parse();
		//return processed object
		return $p->xmlObj;
	}

	/**
	 * Load XML file and return parsed object
	 *
	 * @param int user id
	 * @param int list id
	 * @return mixed representing XML data
	 */
	function grabBuddyObject($uid, $id) {
		//load/download the xml file and parse
		$p = &new gm_xmlParse();

		$this->checkBuddyCache($uid,$id);
		$p->setInputFile($this->GM_CACHE_DIR.md5($uid.'.'.$id).'-buddy.xml');
		$result=$p->parse();
		//return processed object
		return $p->xmlObj;
	}

	/**
	 * Render HTML with buddy list data
	 *
	 * @param int user id
	 * @param int list id
	 * @param string template to use
	 * @return string containing rendered HTML
	 */
	function renderBuddyHTML($uid, $id=0, $template='gm-buddy-default') {
		//get the buddylist object
		$buddyObj=$this->grabBuddyObject($uid,$id);

		if (isset($buddyObj)) {
			//load templates
			$template_layout=$this->loadFile($this->GM_DATA_DIR.$template.'.html');
			$template_player=$this->loadFile($this->GM_DATA_DIR.$template.'-player.html');

			$buddyList=$buddyObj->BUDDYLIST->BUDDY;

			$ptxt='';
			//render player list
			for ($i=0; $i<sizeof($buddyList); $i++) {
				//convert player variables then render it using the template
				$CbuddyList=$this->convertObject($buddyList[$i]);
				$ptxt.=str_replace($CbuddyList->from, $CbuddyList->to, $template_player);
			}

			//add player list render for rendering in layout
			$CbuddyListInfo->from[]='%buddyList%';
			$CbuddyListInfo->to[]=$ptxt;

			//render layout
			return str_replace($CbuddyListInfo->from, $CbuddyListInfo->to, $template_layout);
		} else {
			//failed grabbing object
			return '';
		}
	}

	
	/**
	 * Render HTML with game server data
	 *
	 * @param string ip address of game server
	 * @param int port number of game server
	 * @param string template to use
	 * @return string containing rendered HTML
	 */
	function renderServerHTML($ip, $gamePort, $template='gm-server-default') {
		//get the server object
		$serverObj=$this->grabServerObject($ip,$gamePort);

		if (isset($serverObj)) {
			$playerList=$serverObj->SERVER->PLAYERS->PLAYER;

			//load templates
			$template_layout=$this->loadFile($this->GM_DATA_DIR.$template.'.html');
			$template_player=$this->loadFile($this->GM_DATA_DIR.$template.'-player.html');

			//convert variables for use in rendering templates
			$CserverInfo=$this->convertObject($serverObj->SERVER);

			$ptxt='';
			//render player list
			for ($i=0; $i<sizeof($playerList); $i++) {
				//convert player variables then render it using the template
				$CplayerList=$this->convertObject($playerList[$i]);
				$ptxt.=str_replace($CplayerList->from, $CplayerList->to, $template_player);
			}

			//add player list render for rendering in layout
			$CserverInfo->from[]='%playerList%';
			$CserverInfo->to[]=$ptxt;
		
			//render layout	
			return str_replace($CserverInfo->from, $CserverInfo->to, $template_layout);
		} else {
			//failed grabbing object
			return '';
		}
	}

	/**
	 * Converts HEX color code for use with image
	 *
	 * @param resource image
	 * @param string HEX color code
	 * @return int allocated color
	 */
	function getImageColor($im, $hex) {
		//array of previously allocated colors
		static $colors;

		$retColor='';
		//check if color has been used previously
		for ($i=0; $i<sizeof($colors) && $retColor==''; $i++) {
			if ($colors[$i]->hex==$hex) $retColor=$colors[$i]->color;
		}
		//color has not been used previously
		if ($retColor=='') {
			//get R,G,B values from hex
		        $r=hexdec(substr($hex,0,2));
		        $g=hexdec(substr($hex,2,2));
		        $b=hexdec(substr($hex,4,2));
			
			$newC->hex=$hex;
			//get new color
			$retColor=imagecolorallocate($im,$r,$g,$b);
			$newC->color=$retColor;
			//add color to array
			$colors[]=$newC;
		}
		//return color
		return $retColor;
	}

	/**
	 * Render image with server data
	 *
	 * @param string ip address of game server
	 * @param int port number of game server
	 * @param string ini config file to use
	 * @return resource rendered image
	 */
	function renderServerImage($ip, $gamePort, $ini='gm-sigimage-default.ini') {
		$blankImage=false;

		//get the server object
		$serverObj=$this->grabServerObject($ip,$gamePort);

		if (isset($serverObj)) {
			//load ini file with appearance settings
			$cfg=@parse_ini_file($this->GM_DATA_DIR.$ini,true);
			//convert server object for use
			$CserverInfo=$this->convertObject($serverObj->SERVER);

			//if image is specified		
			if (isset($cfg['image']) && strtolower($cfg['image'])!='blank') {
				$blks=explode('.',$cfg['image']);
				$ext=$blks[sizeof($blks)-1];
				if ($ext=='png') {
					$im=imagecreatefrompng($this->GM_DATA_DIR.$cfg['image']);
				} else if ($ext=='jpg') {
					$im=imagecreatefromjpeg($this->GM_DATA_DIR.$cfg['image']);
				} else {
					//format not recognized, so use blank image instead
					$blankImage=true;
				}
				//if failed loading image
				if (!$im) $blankImage=true;
			} else {
				//use blank image
				$blankImage=true;
			}

			if ($blankImage==true) {
				//check for valid widht/height values in ini file
				if($cfg['width']+0>0 && $cfg['width']+0<=1024 && $cfg['height']+0>0 && $cfg['height']+0<=768) {
					//use those values if valid
					$img_width=$cfg['width']+0;
					$img_height=$cfg['height']+0;
				} else {
					//otherwise default to 468,60
					$img_width=468;
					$img_height=60;
				}
				//create blank image
				$im=imagecreate($img_width,$img_height);
			}
			
			//parse through the whole ini file for o_ values (format=x,y,font size, color)
			while (list($key, $value) = @each ($cfg)) {
				if (substr($key,0,2)=='o_') {
					//grab id value from key string
					$id=substr($key,2)+0;

					//parse individual 
					$blks=explode(',',$value);
					$o_x=$blks[0]+0;
					$o_y=$blks[1]+0;
					$o_f=$blks[2]+0;
					//render text with server data
					$o_text=str_replace($CserverInfo->from,$CserverInfo->to,$cfg['o_text_'.$id]);
					
					//default color = black
					$o_color='000000';
					if (trim($blks[3])!='')
						$o_color=trim($blks[3]);
		
					//place string on image
					imagestring($im,$o_f,$o_x,$o_y,$o_text, $this->getImageColor($im,$o_color));
				}
			}				

		}
		return $im;
	}
}

?>
